interface ITaskEventHeader<TaskMeta> {
    readonly taskId: string;
    readonly meta: TaskMeta;
    readonly timestamp: number;
    readonly performanceNow: number;
}
interface ITaskEventBegin<TaskMeta> extends ITaskEventHeader<TaskMeta> {
    readonly kind: "begin";
}
interface ITaskEventEnd<TaskMeta> extends ITaskEventHeader<TaskMeta> {
    readonly kind: "end";
    readonly duration: number;
    readonly result: {
        readonly kind: "value";
        readonly value: unknown;
    } | {
        readonly kind: "error";
        readonly error: Error;
    };
}
export declare type TaskEvent<TaskMeta> = ITaskEventBegin<TaskMeta> | ITaskEventEnd<TaskMeta>;
declare type TaskListener<TaskMeta> = (event: TaskEvent<TaskMeta>) => void;
interface ITaskLogProps {
    readonly performanceNow: () => number;
    readonly createUniqueId: () => string;
}
export declare class TaskLog<TaskMeta> {
    private readonly props;
    private readonly listeners;
    constructor(props: ITaskLogProps);
    private readonly emit;
    readonly addListener: (listener: TaskListener<TaskMeta>) => TaskListener<TaskMeta>;
    readonly removeListener: (listener: TaskListener<TaskMeta>) => void;
    readonly removeAllListeners: () => void;
    readonly task: <T>(meta: TaskMeta, fn: (taskId: string) => Promise<T>) => Promise<T>;
}
export {};
