"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TaskLog = void 0;
class TaskLog {
    constructor(props) {
        this.emit = (event) => {
            for (const listener of this.listeners) {
                listener(event);
            }
        };
        this.addListener = (listener) => {
            this.listeners.add(listener);
            return listener;
        };
        this.removeListener = (listener) => {
            this.listeners.delete(listener);
        };
        this.removeAllListeners = () => {
            this.listeners.clear();
        };
        this.task = async (meta, fn) => {
            const taskId = this.props.createUniqueId();
            const beginTimestamp = Date.now();
            const beginPerformanceNow = this.props.performanceNow();
            this.emit({
                taskId,
                kind: "begin",
                timestamp: beginTimestamp,
                performanceNow: beginPerformanceNow,
                meta,
            });
            try {
                const value = await fn(taskId);
                const endTimestamp = Date.now();
                const endPerformanceNow = this.props.performanceNow();
                this.emit({
                    taskId,
                    meta,
                    timestamp: endTimestamp,
                    performanceNow: endPerformanceNow,
                    duration: endPerformanceNow - beginPerformanceNow,
                    kind: "end",
                    result: {
                        kind: "value",
                        value,
                    },
                });
                return value;
            }
            catch (error) {
                const endTimestamp = Date.now();
                const endPerformanceNow = this.props.performanceNow();
                this.emit({
                    taskId,
                    meta,
                    timestamp: endTimestamp,
                    performanceNow: endPerformanceNow,
                    duration: endPerformanceNow - beginPerformanceNow,
                    kind: "end",
                    result: {
                        kind: "error",
                        error,
                    },
                });
                throw error;
            }
        };
        this.props = props;
        this.listeners = new Set();
    }
}
exports.TaskLog = TaskLog;
//# sourceMappingURL=TaskLog.js.map